#!/bin/bash 
# (c) Vlad Estivill-Castro
# setting up a clfsm machine for catkin
echo -e "\n(c)2021-2025 Vlad Estivill-Castro\n"
echo -e "Set up for " $1 "on" $(date +"%x %r %Z\n") 
#
# Do we have the .machine in the input
machineName=$1
if [ "$machineName" != "${machineName/machine/}" ]; then
	#echo "extension provided"
machineExtension=$(echo $machineName | sed "s?/??" )
else
	#echo "appending extension"
	machineExtension=$1".machine"
fi
machineName=$(echo $machineExtension | sed "s/\.machine//" )
########## We have the machineName and the machineEstension
UNAME_S=$(uname -s)
UNAME_M=$(uname -m)
thePrefix="lib"
    realtive_colcon_destiantion="../../../install/$machineName" 
    colcon_destination=$(pwd)"/$realtive_colcon_destiantion/lib"
    echo "The destination will become $colcon_destination"
##############
if [ -d $machineExtension ]; then
    echo -e "Machine *>" $machineExtension "<* found. Things are fine.\n\n" 
    outputCMakeLists=Amet_Suggested_CMakeLists_$machineName.txt
    destination=$colcon_destination/$machineExtension/${UNAME_S}-${UNAME_M}
    ############################ create the directory for execution
    if [ -d $destination ]; then
	    echo -e "Destination $destination exists\n"
    else
	    echo -e "Creating destination $destination \n"
	    mkdir -p $destination
    fi
    ########
    # copy include files
    echo "re-creating ../include and ../src"
    mkdir -p ../include
    mkdir -p ../src
    cp $machineExtension/*.h ../include
    cp $machineExtension/*.expr ../include
    cp $machineExtension/*VarRefs.mm ../include
    cp $machineExtension/*OnEntry.mm ../include
    cp $machineExtension/*OnExit.mm ../include
    cp $machineExtension/*Internal.mm ../include
    echo "Your CMakeLists.txt needs : find_package(ament_cmake REQUIRED COMPONENTS clfsm libclfsm)" 
    #
    echo "cmake_minimum_required(VERSION 3.8)" >  $outputCMakeLists
    echo "project("$machineName")" >> $outputCMakeLists
    echo "">> $outputCMakeLists
    echo "if(CMAKE_COMPILER_IS_GNUCXX OR CMAKE_CXX_COMPILER_ID MATCHES" '"Clang")'  >> $outputCMakeLists
    echo "  add_compile_options(-Wall -Wextra -Wpedantic)"  >> $outputCMakeLists
    echo "endif()"  >> $outputCMakeLists
    echo "">> $outputCMakeLists
    echo "# find dependencies" >> $outputCMakeLists
    echo "find_package(ament_cmake REQUIRED)" >> $outputCMakeLists
    echo "# uncomment the following section in order to fill in" >> $outputCMakeLists
    echo "# further dependencies manually." >> $outputCMakeLists
    echo "# IMPORTANT for lfsm packages." >> $outputCMakeLists
    echo "# find_package(<dependency> REQUIRED)" >> $outputCMakeLists
    echo "" >> $outputCMakeLists
    grep "<depend>" ../package.xml | grep "</depend>" | sed 's/.*<depend>/find_package(/' | sed 's?</depend>? REQUIRED)?' >> $outputCMakeLists
    echo "" >> $outputCMakeLists
    echo "" >> $outputCMakeLists
    listOfDependencies=$(grep "<depend>" ../package.xml | grep "</depend>" | sed 's/.*<depend>/ /' | sed 's?</depend>? ?')
    echo "List of dependencies " $listOfDependencies
    #
    echo "###########" >> $outputCMakeLists
    echo "## Build ##"  >> $outputCMakeLists
    echo "###########" >> $outputCMakeLists
    #
    echo "Placing include_directories directives"
    echo "include_directories(include)" >> $outputCMakeLists
    echo 'include_directories(${libclfsm_INCLUDE_DIRS})' >> $outputCMakeLists
    echo 'include_directories(${clfsm_INCLUDE_DIRS})' >>  $outputCMakeLists
    echo "" >> $outputCMakeLists
    echo "" >> $outputCMakeLists
    ########
    echo "## The library below requires several steps " >>  $outputCMakeLists
    echo "## Declare a cpp library" >>  $outputCMakeLists
    echo "# add_library("$machineName >> $outputCMakeLists
    echo "#   src/\${PROJECT_NAME}/"$machineName".cpp" >> $outputCMakeLists
    echo "# )" >> $outputCMakeLists
    # copy source files files
    echo "copying files to target src directory and placing  add_library directives"
    #first the machine
    cp $machineExtension/$machineName.mm ../src
    echo "## First step, the states " >>  $outputCMakeLists
    linkForMachine="add_library($thePrefix$machineName SHARED \n"
    ###### loop over the machine's states
    lineNum=1
    stateName=$(sed -n "${lineNum}p" $machineExtension/States)
    filenameForState=State_$stateName.mm
    while [ -f $machineExtension/$filenameForState ]; do
       echo  "Procesing state :" $stateName
       echo "add_library(${machineName}_State_"$stateName" OBJECT src/"$filenameForState  ")"  >>  $outputCMakeLists
       cp $machineExtension/$filenameForState ../src
       linkForMachine=$linkForMachine'$<TARGET_OBJECTS:'"${machineName}_State_$stateName>\n"
       # update the filename or the state
       lineNum=$((lineNum+1))
       stateName=$(sed -n "${lineNum}p" $machineExtension/States)
       filenameForState=State_$stateName.mm
    done
    #
    # command for the machine
    echo "Placing  add_library($machineName OBJECT src/$machineName.mm )"
    echo "add_library($machineName OBJECT src/$machineName.mm )" >>  $outputCMakeLists
    linkForMachine=$linkForMachine'$<TARGET_OBJECTS:'$machineName">)"
    echo -e $linkForMachine >> $outputCMakeLists
    echo "set_target_properties($thePrefix$machineName  PROPERTIES OUTPUT_NAME $machineName)" >> $outputCMakeLists
    #
    # repeat the loop over the states for Linux additional flags
    #
    echo "############### On Linux .mm files must be understood as C++" >> $outputCMakeLists
    echo 'if(${CMAKE_SYSTEM_NAME} MATCHES "Linux")' >> $outputCMakeLists
    lineNum=1
    stateName=$(sed -n "${lineNum}p" $machineExtension/States)
    filenameForState=State_$stateName.mm
    while [ -f $machineExtension/$filenameForState ]; do
	    echo "set_target_properties(${machineName}_State_$stateName PROPERTIES COMPILE_FLAGS" '"-x c++ -fPIC")' >> $outputCMakeLists
       # update the filename or the state
       lineNum=$((lineNum+1))
       stateName=$(sed -n "${lineNum}p" $machineExtension/States)
       filenameForState=State_$stateName.mm
    done
    #
    echo "Your CMakeLists.txt needs : set_target_properties($machineName PROPERTIES COMPILE_FLAGS" '"-x c++ -fPIC")'
    echo "set_target_properties($machineName PROPERTIES COMPILE_FLAGS" '"-x c++ -fPIC")' >> $outputCMakeLists
    #
    echo 'endif(${CMAKE_SYSTEM_NAME} MATCHES "Linux")' >> $outputCMakeLists
    ##########
    echo "" >> $outputCMakeLists
    echo "target_link_libraries($thePrefix$machineName " '${libclfsm_LIBRARIES} ${catkin_LIBRARIES})'
    echo "target_link_libraries($thePrefix$machineName " '${libclfsm_LIBRARIES} ${catkin_LIBRARIES})'  >> $outputCMakeLists
    echo "" >> $outputCMakeLists
    ##########
    # Need to indicate the dependencies to all compiled parts
    ##########
    echo "ament_target_dependencies(lib"$machineName " "$listOfDependencies")" >> $outputCMakeLists
    echo "ament_target_dependencies("$machineName " "$listOfDependencies")" >> $outputCMakeLists
     ###### loop over the machine's states
    lineNum=1
    stateName=$(sed -n "${lineNum}p" $machineExtension/States)
    filenameForState=State_$stateName.mm
    while [ -f $machineExtension/$filenameForState ]; do
	    echo "ament_target_dependencies(${machineName}_State_"$stateName " "$listOfDependencies")" >> $outputCMakeLists
       # update the filename or the state
       lineNum=$((lineNum+1))
       stateName=$(sed -n "${lineNum}p" $machineExtension/States)
       filenameForState=State_$stateName.mm
    done
    ##########
    echo -e "\nIMPOTRANT ==> TO BUILD, [if dependencies are all included] "
    echo -e "cp $outputCMakeLists  ../CMakeLists.txt"
    echo -e "In another terminal at the root of the workspace"
    echo -e "colcon build --packages-select "${machineName}
    ##########
    echo "#############" >> $outputCMakeLists
    echo "## Install ##" >> $outputCMakeLists
    echo "#############" >> $outputCMakeLists
    echo "" >> $outputCMakeLists
    echo "" >> $outputCMakeLists
    ##########
    echo "## Mark executables and/or libraries for installation" >> $outputCMakeLists
    echo "install(TARGETS "lib$machineName  >> $outputCMakeLists
    echo "   LIBRARY DESTINATION \${CATKIN_PACKAGE_LIB_DESTINATION}" >> $outputCMakeLists
    echo " )" >> $outputCMakeLists
    echo "" >> $outputCMakeLists
    echo "ament_export_libraries(lib"$machineName")" >> $outputCMakeLists
    echo "" >> $outputCMakeLists
    echo "ament_package()" >> $outputCMakeLists
    ##########

    ##########
    # Hints for installing
    echo -e "\nIMPOTRANT ==> TO FINISH  INSTALLING: $machineName.machine \n"
    echo "You should copy the result after a succesfull colcon build"
    echo "You are at $(pwd)":
    if [ $UNAME_S = "Darwin" ]; then
       result="$realtive_colcon_destiantion/lib/lib$machineName.dylib"
       else
       result="$realtive_colcon_destiantion/lib/lib$machineName.so"
       fi
    echo -e "The compiled result should be at $result\n"
    echo -e "mv $result $destination/$machineName.so\n"
    echo -e "\nRUNNING without ros: $machineName.machine \n"
    echo -e "../../../install/clfsm/lib/clfsm/clfsm $realtive_colcon_destiantion/lib/$machineName.machine \n"
    #
else
       echo "Machine not found! Run this script in the same directory where <machineName>.machine is "
fi
